@@ -249,11 +249,22 @@ class Graph(Magics):
                             choices=['dynamic', 'static', 'details'])
         parser.add_argument('--explain-format', default='text/html', help='response format for explain query mode',
                             choices=['text/csv', 'text/html'])
+        parser.add_argument('-g', '--group-by', type=str, default='',
+                            help='Property used to group nodes.')
+        parser.add_argument('-d', '--display-property', type=str, default='',
+                            help='Property to display the value of on each node.')
+        parser.add_argument('-de', '--edge-display-property', type=str, default='',
+                            help='Property to display the value of on each edge.')
+        parser.add_argument('-t', '--tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip.')
+        parser.add_argument('-te', '--edge-tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip.')
         parser.add_argument('-l', '--label-max-length', type=int, default=10,
                             help='Specifies max length of vertex labels, in characters. Default is 10')
         parser.add_argument('-le', '--edge-label-max-length', type=int, default=10,
                             help='Specifies max length of edge labels, in characters. Default is 10')
         parser.add_argument('--store-to', type=str, default='', help='store query result to this variable')
+        parser.add_argument('--ignore-groups', action='store_true', default=False, help="Ignore all grouping options")
         parser.add_argument('-sp', '--stop-physics', action='store_true', default=False,
                             help="Disable visualization physics after the initial simulation stabilizes.")
         parser.add_argument('-sd', '--simulation-duration', type=int, default=1500,
@@ -309,8 +320,16 @@ class Graph(Magics):
                     sparql_metadata = build_sparql_metadata_from_query(query_type='query', res=query_res,
                                                                        results=results, scd_query=True)
 
-                    sn = SPARQLNetwork(label_max_length=args.label_max_length,
-                                       edge_label_max_length=args.edge_label_max_length, expand_all=args.expand_all)
+                    sn = SPARQLNetwork(group_by_property=args.group_by,
+                                       display_property=args.display_property,
+                                       edge_display_property=args.edge_display_property,
+                                       tooltip_property=args.tooltip_property,
+                                       edge_tooltip_property=args.edge_tooltip_property,
+                                       label_max_length=args.label_max_length,
+                                       edge_label_max_length=args.edge_label_max_length,
+                                       ignore_groups=args.ignore_groups,
+                                       expand_all=args.expand_all)
+                    
                     sn.extract_prefix_declarations_from_query(cell)
                     try:
                         sn.add_results(results)
@@ -424,6 +443,12 @@ class Graph(Magics):
                             help='Property to display the value of on each node, default is T.label')
         parser.add_argument('-de', '--edge-display-property', type=str, default='T.label',
                             help='Property to display the value of on each edge, default is T.label')
+        parser.add_argument('-t', '--tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip. If not specified, tooltip '
+                                 'will default to the node label value.')
+        parser.add_argument('-te', '--edge-tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip. If not specified, tooltip '
+                                 'will default to the edge label value.')
         parser.add_argument('-l', '--label-max-length', type=int, default=10,
                             help='Specifies max length of vertex label, in characters. Default is 10')
         parser.add_argument('-le', '--edge-label-max-length', type=int, default=10,
@@ -508,6 +533,8 @@ class Graph(Magics):
                     logger.debug(f'ignore_groups: {args.ignore_groups}')
                     gn = GremlinNetwork(group_by_property=args.group_by, display_property=args.display_property,
                                         edge_display_property=args.edge_display_property,
+                                        tooltip_property=args.tooltip_property,
+                                        edge_tooltip_property=args.edge_tooltip_property,
                                         label_max_length=args.label_max_length,
                                         edge_label_max_length=args.edge_label_max_length,
                                         ignore_groups=args.ignore_groups)
@@ -781,10 +808,20 @@ class Graph(Magics):
         # TODO: change widgets to let any arbitrary inputs be added by users
         parser = argparse.ArgumentParser()
         parser.add_argument('-s', '--source', default='s3://')
-        parser.add_argument('-l', '--loader-arn', default=self.graph_notebook_config.load_from_s3_arn)
+        try:
+            parser.add_argument('-l', '--loader-arn', default=self.graph_notebook_config.load_from_s3_arn)
+        except AttributeError:
+            print(f"Missing required configuration option 'load_from_s3_arn'. Please ensure that you have provided a "
+                  "valid Neptune cluster endpoint URI in the 'host' field of %graph_notebook_config.")
+            return
         parser.add_argument('-f', '--format', choices=LOADER_FORMAT_CHOICES, default=FORMAT_CSV)
         parser.add_argument('-p', '--parallelism', choices=PARALLELISM_OPTIONS, default=PARALLELISM_HIGH)
-        parser.add_argument('-r', '--region', default=self.graph_notebook_config.aws_region)
+        try:
+            parser.add_argument('-r', '--region', default=self.graph_notebook_config.aws_region)
+        except AttributeError:
+            print("Missing required configuration option 'aws_region'. Please ensure that you have provided a "
+                  "valid Neptune cluster endpoint URI in the 'host' field of %graph_notebook_config.")
+            return
         parser.add_argument('--fail-on-failure', action='store_true', default=False)
         parser.add_argument('--update-single-cardinality', action='store_true', default=True)
         parser.add_argument('--store-to', type=str, default='', help='store query result to this variable')
@@ -1168,9 +1205,9 @@ class Graph(Magics):
                         kwargs['parserConfiguration']['baseUri'] = base_uri.value
 
                 if source.value.startswith("s3://"):
-                    load_res = self.client.load(source.value, source_format.value, arn.value, **kwargs)
+                    load_res = self.client.load(str(source_exp), source_format.value, arn.value, **kwargs)
                 else:
-                    load_res = self.client.load(source.value, source_format.value, **kwargs)
+                    load_res = self.client.load(str(source_exp), source_format.value, **kwargs)
                 load_res.raise_for_status()
                 load_result = load_res.json()
                 store_to_ns(args.store_to, load_result, local_ns)
@@ -1550,6 +1587,12 @@ class Graph(Magics):
                             help='Property to display the value of on each node, default is ~labels')
         parser.add_argument('-de', '--edge-display-property', type=str, default='~labels',
                             help='Property to display the value of on each edge, default is ~type')
+        parser.add_argument('-t', '--tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip. If not specified, tooltip '
+                                 'will default to the node label value.')
+        parser.add_argument('-te', '--edge-tooltip-property', type=str, default='',
+                            help='Property to display the value of on each node tooltip. If not specified, tooltip '
+                                 'will default to the edge label value.')
         parser.add_argument('-l', '--label-max-length', type=int, default=10,
                             help='Specifies max length of vertex label, in characters. Default is 10')
         parser.add_argument('-rel', '--rel-label-max-length', type=int, default=10,
@@ -1583,8 +1626,10 @@ class Graph(Magics):
                 try:
                     gn = OCNetwork(group_by_property=args.group_by, display_property=args.display_property,
                                    edge_display_property=args.edge_display_property,
+                                   tooltip_property=args.tooltip_property,
+                                   edge_tooltip_property=args.edge_tooltip_property,
                                    label_max_length=args.label_max_length,
-                                   rel_label_max_length=args.rel_label_max_length,
+                                   edge_label_max_length=args.rel_label_max_length,
                                    ignore_groups=args.ignore_groups)
                     gn.add_results(res)
                     logger.debug(f'number of nodes is {len(gn.graph.nodes)}')
@@ -1649,7 +1694,7 @@ class Graph(Magics):
 
     def handle_opencypher_status(self, line, local_ns):
         """
-        This is refactored into its own handler method so that we can invoke is from
+        This is refactored xinto its own handler method so that we can invoke is from
         %opencypher_status or from %oc_status
         """
         parser = argparse.ArgumentParser()
